@@ -438,9 +438,11 @@ void FFmpegCapture::send_disconnected_frame()
 			memset(video_frame.data + frame_width * frame_height, 128, frame_width * frame_height);  // Valid for both NV12 and planar.
 		}
 
-		frame_callback(-1, AVRational{1, TIMEBASE}, -1, AVRational{1, TIMEBASE}, timecode++,
-			video_frame, /*video_offset=*/0, video_format,
-			FrameAllocator::Frame(), /*audio_offset=*/0, AudioFormat());
+		if (frame_callback != nullptr) {
+			frame_callback(-1, AVRational{1, TIMEBASE}, -1, AVRational{1, TIMEBASE}, timecode++,
+				video_frame, /*video_offset=*/0, video_format,
+				FrameAllocator::Frame(), /*audio_offset=*/0, AudioFormat());
+		}
 		last_frame_was_connected = false;
 	}
 
@@ -734,9 +736,11 @@ bool FFmpegCapture::play_video(const string &pathname)
 					timecode += MAX_FPS * 2 + 1;
 				}
 				last_neutral_color = get_neutral_color(frame->metadata);
-				frame_callback(frame->pts, video_timebase, audio_pts, audio_timebase, timecode++,
-					video_frame.get_and_release(), 0, video_format,
-					audio_frame.get_and_release(), 0, audio_format);
+				if (frame_callback != nullptr) {
+					frame_callback(frame->pts, video_timebase, audio_pts, audio_timebase, timecode++,
+						video_frame.get_and_release(), 0, video_format,
+						audio_frame.get_and_release(), 0, audio_format);
+				}
 				first_frame = false;
 				last_frame = steady_clock::now();
 				last_frame_was_connected = true;
@@ -841,13 +845,16 @@ AVFrameWithDeleter FFmpegCapture::decode_frame(AVFormatContext *format_ctx, AVCo
 			if (pkt.stream_index == audio_stream_index && audio_callback != nullptr) {
 				audio_callback(&pkt, format_ctx->streams[audio_stream_index]->time_base);
 			}
-			if (pkt.stream_index == video_stream_index) {
+			if (pkt.stream_index == video_stream_index && video_callback != nullptr) {
+				video_callback(&pkt, format_ctx->streams[video_stream_index]->time_base);
+			}
+			if (pkt.stream_index == video_stream_index && global_flags.transcode_video) {
 				if (avcodec_send_packet(video_codec_ctx, &pkt) < 0) {
 					fprintf(stderr, "%s: Cannot send packet to video codec.\n", pathname.c_str());
 					*error = true;
 					return AVFrameWithDeleter(nullptr);
 				}
-			} else if (pkt.stream_index == audio_stream_index) {
+			} else if (pkt.stream_index == audio_stream_index && global_flags.transcode_audio) {
 				has_audio = true;
 				if (avcodec_send_packet(audio_codec_ctx, &pkt) < 0) {
 					fprintf(stderr, "%s: Cannot send packet to audio codec.\n", pathname.c_str());
@@ -1109,7 +1116,7 @@ unsigned FFmpegCapture::frame_height(const AVFrame *frame) const
 	if (height == 0) {
 		return frame->height;
 	} else {
-		return width;
+		return height;
 	}
 }
 
