@@ -234,6 +234,16 @@ void JitterHistory::unregister_metrics(const vector<pair<string, string>> &label
 
 void JitterHistory::frame_arrived(steady_clock::time_point now, int64_t frame_duration, size_t dropped_frames)
 {
+	if (frame_duration != last_duration) {
+		// If the frame rate changed, the input clock is also going to change,
+		// so our historical data doesn't make much sense anymore.
+		// Also, format changes typically introduce blips that are not representative
+		// of the typical frame stream. (We make the assumption that format changes
+		// don't happen all the time in regular use; if they did, we should probably
+		// rather keep the history so that we take jitter they may introduce into account.)
+		clear();
+		last_duration = frame_duration;
+	}
 	if (expected_timestamp > steady_clock::time_point::min()) {
 		expected_timestamp += dropped_frames * nanoseconds(frame_duration * 1000000000 / TIMEBASE);
 		double jitter_seconds = fabs(duration<double>(expected_timestamp - now).count());
@@ -277,7 +287,7 @@ void QueueLengthPolicy::unregister_metrics(const vector<pair<string, string>> &l
 }
 
 void QueueLengthPolicy::update_policy(steady_clock::time_point now,
-                                      steady_clock::time_point expected_next_frame,
+                                      steady_clock::time_point expected_next_input_frame,
                                       int64_t input_frame_duration,
                                       int64_t master_frame_duration,
                                       double max_input_card_jitter_seconds,
@@ -288,7 +298,7 @@ void QueueLengthPolicy::update_policy(steady_clock::time_point now,
 
 	// Figure out when we can expect the next frame for this card, assuming
 	// worst-case jitter (ie., the frame is maximally late).
-	double seconds_until_next_frame = max(duration<double>(expected_next_frame - now).count() + max_input_card_jitter_seconds, 0.0);
+	double seconds_until_next_frame = max(duration<double>(expected_next_input_frame - now).count() + max_input_card_jitter_seconds, 0.0);
 
 	// How many times are the master card expected to tick in that time?
 	// We assume the master clock has worst-case jitter but not any rate
@@ -419,7 +429,7 @@ Mixer::Mixer(const QSurfaceFormat &format)
 
 				DeckLinkCapture *capture = new DeckLinkCapture(decklink, card_index);
 				DeckLinkOutput *output = new DeckLinkOutput(resource_pool.get(), decklink_output_surface, global_flags.width, global_flags.height, card_index);
-				if (!output->set_device(decklink)) {
+				if (!output->set_device(decklink, capture->get_input())) {
 					delete output;
 					output = nullptr;
 				}
@@ -499,10 +509,6 @@ Mixer::Mixer(const QSurfaceFormat &format)
 	}
 #endif
 
-	for (unsigned card_index = 0; card_index < MAX_VIDEO_CARDS; ++card_index) {
-		cards[card_index].queue_length_policy.reset(card_index);
-	}
-
 	chroma_subsampler.reset(new ChromaSubsampler(resource_pool.get()));
 
 	if (global_flags.ten_bit_input) {
@@ -890,7 +896,7 @@ void Mixer::set_output_card_internal(int card_index)
 		card->parked_capture = move(card->capture);
 		CaptureInterface *fake_capture = new FakeCapture(global_flags.width, global_flags.height, FAKE_FPS, OUTPUT_FREQUENCY, card_index, global_flags.fake_cards_audio);
 		configure_card(card_index, fake_capture, CardType::FAKE_CAPTURE, card->output.release(), /*is_srt_card=*/false);
-		card->queue_length_policy.reset(card_index);
+		card->jitter_history.clear();
 		card->capture->start_bm_capture();
 		desired_output_video_mode = output_video_mode = card->output->pick_video_mode(desired_output_video_mode);
 		card->output->start_output(desired_output_video_mode, pts_int);
@@ -1608,7 +1614,7 @@ void Mixer::handle_hotplugged_cards()
 				fprintf(stderr, "Card %u went away, replacing with a fake card.\n", card_index);
 				FakeCapture *capture = new FakeCapture(global_flags.width, global_flags.height, FAKE_FPS, OUTPUT_FREQUENCY, card_index, global_flags.fake_cards_audio);
 				configure_card(card_index, capture, CardType::FAKE_CAPTURE, /*output=*/nullptr, /*is_srt_card=*/false);
-				card->queue_length_policy.reset(card_index);
+				card->jitter_history.clear();
 				card->capture->start_bm_capture();
 			} else {
 				// NOTE: The theme might end up forcing the card back at some later point
@@ -1618,7 +1624,7 @@ void Mixer::handle_hotplugged_cards()
 				fprintf(stderr, "Card %u went away, removing. (To keep a fake card, increase --num-cards.)\n", card_index);
 				theme->remove_card(card_index);
 				configure_card(card_index, /*capture=*/nullptr, CardType::FAKE_CAPTURE, /*output=*/nullptr, /*is_srt_card=*/false);
-				card->queue_length_policy.reset(card_index);
+				card->jitter_history.clear();
 			}
 		}
 	}
@@ -1665,7 +1671,7 @@ void Mixer::handle_hotplugged_cards()
 			CaptureCard *card = &cards[free_card_index];
 			BMUSBCapture *capture = new BMUSBCapture(free_card_index, new_dev);
 			configure_card(free_card_index, capture, CardType::LIVE_CARD, /*output=*/nullptr, /*is_srt_card=*/false);
-			card->queue_length_policy.reset(free_card_index);
+			card->jitter_history.clear();
 			capture->set_card_disconnected_callback(bind(&Mixer::bm_hotplug_remove, this, free_card_index));
 			capture->start_bm_capture();
 		}
@@ -1721,7 +1727,7 @@ void Mixer::handle_hotplugged_cards()
 			configure_card(free_card_index, capture, CardType::FFMPEG_INPUT, /*output=*/nullptr, /*is_srt_card=*/true);
 			update_srt_stats(sock, card);  // Initial zero stats.
 			card->last_srt_stream_id = stream_id;
-			card->queue_length_policy.reset(free_card_index);
+			card->jitter_history.clear();
 			capture->set_card_disconnected_callback(bind(&Mixer::bm_hotplug_remove, this, free_card_index));
 			capture->start_bm_capture();
 		}
@@ -1734,7 +1740,7 @@ void Mixer::handle_hotplugged_cards()
 		if (card->capture == nullptr && card->force_active) {
 			FakeCapture *capture = new FakeCapture(global_flags.width, global_flags.height, FAKE_FPS, OUTPUT_FREQUENCY, card_index, global_flags.fake_cards_audio);
 			configure_card(card_index, capture, CardType::FAKE_CAPTURE, /*output=*/nullptr, /*is_srt_card=*/false);
-			card->queue_length_policy.reset(card_index);
+			card->jitter_history.clear();
 			card->capture->start_bm_capture();
 		}
 	}
@@ -1980,7 +1986,7 @@ void Mixer::quit()
 	if (global_flags.srt_port >= 0) {
 		// There's seemingly no other reasonable way to wake up the thread
 		// (libsrt's epoll equivalent is busy-waiting).
-		int sock = srt_socket(AF_INET6, 0, 0);
+		int sock = srt_create_socket();
 		if (sock != -1) {
 			sockaddr_in6 addr;
 			memset(&addr, 0, sizeof(addr));
@@ -2189,7 +2195,7 @@ void Mixer::OutputChannel::set_color_updated_callback(Mixer::color_updated_callb
 #ifdef HAVE_SRT
 void Mixer::start_srt()
 {
-	SRTSOCKET sock = srt_socket(AF_INET6, 0, 0);
+	SRTSOCKET sock = srt_create_socket();
 	sockaddr_in6 addr;
 	memset(&addr, 0, sizeof(addr));
 	addr.sin6_family = AF_INET6;
