@@ -40,7 +40,7 @@ extern "C" {
 #include <libavcodec/avcodec.h>
 #include <libavformat/avio.h>
 #include <libavutil/error.h>
-#include <libdrm/drm_fourcc.h>
+#include <drm_fourcc.h>
 
 }  // namespace
 
